"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.pgClient = void 0;
const pg_1 = require("pg");
const kysely_1 = require("kysely");
const transferFromDB = (o) => {
    return {
        id: o.id,
        solKey: o.sol_key,
        solTx: o.sol_tx,
        sqTx: o.sq_tx,
        txStatus: o.txStatus,
        createdAt: o.created_at,
        payedAt: o.payed_at,
        amountDue: BigInt(o.amount_due),
        amountPayed: o.amount_payed ? BigInt(o.amount_payed) : undefined,
        operatorRewards: o.transfer_type == 'node_operator_rewards' ? {
            epochId: o.epoch_id,
            startTime: o.start_time,
            endTime: o.end_time
        } : undefined,
        computeClientRewards: o.transfer_type == 'open_compute_rewards' ? {
            epochId: o.epoch_id,
            deviceId: o.device_id
        } : undefined,
        upgradeRewards: o.transfer_type == 'upgrade_rewards' && !!o.epoch_id ? {
            epochId: o.epoch_id,
            epochStartTime: o.epoch_start_time,
            points: BigInt(o.points)
        } : undefined
    };
};
const pgClient = (config) => {
    var _a, _b;
    const dialect = new kysely_1.PostgresDialect({
        pool: new pg_1.Pool(Object.assign(Object.assign({}, config), { port: (_a = config.port) !== null && _a !== void 0 ? _a : 5432, max: (_b = config.max) !== null && _b !== void 0 ? _b : 10 }))
    });
    // Database interface is passed to Kysely's constructor, and from now on, Kysely
    // knows your database structure.
    // Dialect is passed to Kysely's constructor, and from now on, Kysely knows how
    // to communicate with your database.
    const db = new kysely_1.Kysely({
        dialect,
    });
    const TRANSFER = 'transfer';
    const NODE_OPERATOR_REWARDS_TRANSFER = 'node_operator_rewards_transfer';
    const COMPUTE_CLIENT_REWARDS_TRANSFER = 'open_compute_rewards_transfer';
    const UPGRADE_REWARDS_TRANSFER = 'upgrade_rewards_transfer';
    const SQUADS_TX = 'squads_tx';
    const NODE_OPERATOR_EPOCH = 'node_operator_epoch';
    const JOB_ID = 'job_id';
    const createTransfers = (db, transfers) => __awaiter(void 0, void 0, void 0, function* () {
        for (const t of transfers) {
            const { id: transferId } = (yield db.insertInto(TRANSFER).values({
                amount_due: BigInt(t.amountDue),
                amount_payed: t.amountPayed ? BigInt(t.amountPayed) : 0,
                created_at: t.createdAt,
                payed_at: t.payedAt,
                sol_key: t.solKey,
                sol_tx: t.solTx,
                transfer_type: (() => {
                    if (t.operatorRewards) {
                        return 'node_operator_rewards';
                    }
                    else if (t.upgradeRewards) {
                        return 'upgrade_rewards';
                    }
                    else if (t.computeClientRewards) {
                        return 'open_compute_rewards';
                    }
                    throw new Error('wtf');
                })(),
            }).returning((eb) => ['id as id']).executeTakeFirst());
            console.log(`inserted transfer ${JSON.stringify(t, (key, value) => typeof value === 'bigint'
                ? value.toString()
                : value // return everything else unchanged
            )}`);
            if (t.operatorRewards) {
                console.log(`inserting operator rewards ${transferId}`);
                const x = (yield db.insertInto(NODE_OPERATOR_REWARDS_TRANSFER).values({
                    end_time: t.operatorRewards.endTime,
                    start_time: t.operatorRewards.startTime,
                    epoch_id: t.operatorRewards.epochId,
                    transfer_id: transferId,
                }).executeTakeFirst()).insertId;
            }
            else if (t.computeClientRewards) {
                console.log(`inserting compute client rewards ${transferId}`);
                const x = (yield db.insertInto(COMPUTE_CLIENT_REWARDS_TRANSFER).values({
                    device_id: t.computeClientRewards.deviceId,
                    transfer_id: transferId
                }).executeTakeFirst()).insertId;
            }
            else if (t.upgradeRewards) {
                console.log(`inserting upgrade rewards ${transferId}`);
                const x = (yield db.insertInto(UPGRADE_REWARDS_TRANSFER).values({
                    epoch_id: t.upgradeRewards.epochId,
                    epoch_start_time: t.upgradeRewards.epochStartTime,
                    points: t.upgradeRewards.points,
                    transfer_id: transferId
                }).executeTakeFirst()).insertId;
            }
        }
        return;
    });
    return {
        saveNodeOperatorEpoch(epochId, transfers, jobIds) {
            return __awaiter(this, void 0, void 0, function* () {
                yield db.transaction().execute((trx) => __awaiter(this, void 0, void 0, function* () {
                    yield createTransfers(trx, transfers);
                    console.log(`inserted transfers`);
                    for (const id of jobIds) {
                        yield trx.insertInto(JOB_ID).values({
                            id: BigInt(id),
                            epoch_id: epochId,
                        }).executeTakeFirstOrThrow();
                    }
                    console.log(`inserted jobids`);
                    const now = new Date(Date.now());
                    const res = yield trx.updateTable(NODE_OPERATOR_EPOCH)
                        .set({ saved_at: now })
                        .where('id', '=', epochId.toString()).executeTakeFirstOrThrow();
                    console.log(`saved node operator epoch: ${res.numUpdatedRows}`);
                }));
                return;
            });
        },
        getLatestNodeOperatorEpoch() {
            var _a;
            return __awaiter(this, void 0, void 0, function* () {
                const now = new Date(Date.now());
                try {
                    const res = yield db.selectFrom(NODE_OPERATOR_EPOCH)
                        .selectAll()
                        .where('end_time', '<=', now)
                        .orderBy('end_time desc')
                        .limit(1)
                        .executeTakeFirstOrThrow();
                    return {
                        epochId: BigInt(res.id),
                        startedAt: res.start_time,
                        endedAt: res.end_time,
                        savedAt: (_a = res.saved_at) !== null && _a !== void 0 ? _a : undefined
                    };
                }
                catch (e) {
                    if (e instanceof kysely_1.NoResultError) {
                        console.log(`no node operator epoch found`);
                        return undefined;
                    }
                    throw e;
                }
            });
        },
        getOutstanding(f) {
            return __awaiter(this, void 0, void 0, function* () {
                //select sol_key, sum(amount_due)-sum(amount_payed) from transfer group by sol_key;
                const res = yield db.selectFrom(TRANSFER)
                    .select(({ fn, val, ref }) => [
                    'sol_key',
                    (0, kysely_1.sql) `${fn.sum('amount_due')} - ${fn.sum('amount_payed')}`.as('due'),
                ])
                    .where('transfer_type', '=', f.typ)
                    .groupBy(['sol_key'])
                    .execute();
                const out = {};
                for (const r of res) {
                    out[r.sol_key] = BigInt(r.due);
                }
                return out;
            });
        },
        updateSqTx(tx, status) {
            return __awaiter(this, void 0, void 0, function* () {
                const res = yield db.updateTable(SQUADS_TX)
                    .set({ tx_status: status })
                    .where('sq_tx', '=', tx).execute();
                console.log(`updated ${tx} -> ${status}`);
            });
        },
        setSqTx(transferIds, sqTx) {
            return __awaiter(this, void 0, void 0, function* () {
                const { id } = (yield db.insertInto(SQUADS_TX)
                    .values({ sq_tx: sqTx, tx_status: 'created' })
                    .returning((eb) => ['id as id']).executeTakeFirst());
                const res = yield db.updateTable(TRANSFER)
                    .set({ sq_tx_id: id })
                    .where('id', 'in', transferIds.map(id => id.toString())).execute();
                console.log(`set sqTx ${sqTx} on ${transferIds.length} transfers`);
            });
        },
        /*
        [
              'transfer.id',
              'transfer.sol_key',
              'transfer.transfer_type',
              'transfer.transfer_type',
              'squads_tx.tx_status', 'squads_tx.sq_tx']
         */
        fetchTransfers(f) {
            return __awaiter(this, void 0, void 0, function* () {
                let base = db
                    .selectFrom(TRANSFER)
                    // @ts-ignore
                    .selectAll(TRANSFER)
                    .leftJoin(SQUADS_TX, 'squads_tx.id', 'transfer.sq_tx_id')
                    .select(['squads_tx.tx_status', 'squads_tx.sq_tx'])
                    .where('transfer_type', '=', f.typ);
                if (f.status) {
                    if (f.status == 'notx') {
                        base = base.where('squads_tx.tx_status', 'is', null);
                    }
                    else {
                        base = base.where('squads_tx.tx_status', '=', f.status);
                    }
                }
                if (f.payed != undefined) {
                    base = base.where('payed_at', f.payed ? 'is not' : 'is', null);
                }
                if (f.typ == 'upgrade_rewards') {
                    base = base.innerJoin(UPGRADE_REWARDS_TRANSFER, 'upgrade_rewards_transfer.transfer_id', 'transfer.id');
                    if (f.epochId) {
                        // @ts-ignore
                        base = base.where('upgrade_rewards_transfer.epoch_id', '=', f.epochId);
                    }
                }
                if (f.typ == 'node_operator_rewards') {
                    base = base.innerJoin(NODE_OPERATOR_REWARDS_TRANSFER, 'node_operator_rewards_transfer.transfer_id', 'transfer.id');
                    if (f.epochId) {
                        // @ts-ignore
                        base = base.where('node_operator_rewards_transfer.epoch_id', '=', f.epochId);
                    }
                }
                if (f.sort) {
                    base = base.orderBy('transfer.sol_key asc');
                }
                const res = yield base.execute();
                console.log(`fetched ${base.compile().sql} ${res.length} transfers`);
                return res.map(r => transferFromDB(r));
            });
        },
        payoutTransfersSquads(ts, payedAt, solTx, sqTx) {
            return __awaiter(this, void 0, void 0, function* () {
                const { id: sqTxId } = (yield db.insertInto(SQUADS_TX)
                    .values({ sq_tx: sqTx, tx_status: 'executed' })
                    .returning((eb) => ['id as id']).executeTakeFirstOrThrow());
                let numUpdated = BigInt(0);
                for (const [id, amtPayed] of Object.entries(ts)) {
                    const res = yield db.updateTable(TRANSFER).set({
                        payed_at: payedAt,
                        sol_tx: solTx,
                        amount_payed: amtPayed,
                        sq_tx_id: sqTxId
                    }).where('id', '=', id).executeTakeFirstOrThrow();
                    numUpdated += res.numUpdatedRows;
                }
                console.log(`marked ${numUpdated} transfers payed out`);
                return;
            });
        },
        payoutTransfer(id, payedAt, solTx, amtPayed, sqTx) {
            return __awaiter(this, void 0, void 0, function* () {
                const { id: sqTxId } = (yield db.insertInto(SQUADS_TX)
                    .values({ sq_tx: sqTx, tx_status: 'executed' })
                    .onConflict((oc) => oc
                    .column('sq_tx')
                    .doNothing())
                    .returning((eb) => ['id as id']).executeTakeFirst());
                const res = yield db.updateTable(TRANSFER).set({
                    payed_at: payedAt,
                    sol_tx: solTx,
                    amount_payed: amtPayed,
                    sq_tx_id: sqTxId
                })
                    .where('id', '=', id.toString())
                    .executeTakeFirst();
                console.log(`marked ${res.numUpdatedRows} transfer payed out`);
                return;
            });
        },
        createTransfers(transfers) {
            return __awaiter(this, void 0, void 0, function* () {
                yield createTransfers(db, transfers);
            });
        }
    };
};
exports.pgClient = pgClient;
//# sourceMappingURL=pgClient.js.map