"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderClient = void 0;
const renderClient = (authToken) => {
    const host = `rndr.x.io`;
    return {
        fetchNodeOperatorRewards(startTime, endTime) {
            return __awaiter(this, void 0, void 0, function* () {
                const startTimeStr = startTime.toISOString().split('T')[0];
                const endTimeStr = endTime.toISOString().split('T')[0];
                const url = `https://${host}/api/v3/services/solana/payments?finished_after=${startTimeStr}&finished_before=${endTimeStr}`;
                console.log(`fetching ${url}`);
                const res = yield fetch(url, { headers: {
                        Authorization: `${authToken}`,
                        Accept: 'application/vnd.api+json'
                    } }).then((res) => __awaiter(this, void 0, void 0, function* () {
                    if (res.status != 200) {
                        throw new Error(`failed fetching node operator rewards. code: ${res.status} body: ${JSON.stringify(yield res.text())}`);
                    }
                    return (yield res.json());
                }));
                const solKeyToDue = {};
                for (const r of res.data.attributes.paymentsOwed) {
                    const [addr, amt] = Object.entries(r)[0];
                    solKeyToDue[addr] = amt;
                }
                return { solKeyToDue, jobIds: res.meta.jobIds };
            });
        },
        markJobsPayed(ids, finishedAfter, finishedBefore, totalDistributed) {
            return __awaiter(this, void 0, void 0, function* () {
                const payload = {
                    data: {
                        attributes: {
                            finished_after: finishedAfter.toISOString().split('T')[0],
                            finished_before: finishedBefore.toISOString().split('T')[0],
                            job_ids: ids,
                            total_distributed: totalDistributed
                        }
                    }
                };
                const res = yield fetch(`https://${host}/api/v3/services/solana/payments`, {
                    method: 'POST',
                    headers: {
                        'Authorization': `${authToken}`,
                        'Content-Type': `application/json`
                    },
                    body: JSON.stringify(payload)
                }).then((res) => __awaiter(this, void 0, void 0, function* () {
                    if (res.status != 201) {
                        throw new Error(`failed posting node operator rewards. code: ${res.status} body: ${JSON.stringify(yield res.text())}`);
                    }
                    return (yield res.text());
                }));
                console.log(`markJobsPayed ${res}`);
                return;
            });
        }
    };
};
exports.renderClient = renderClient;
//# sourceMappingURL=renderClient.js.map