"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeSqV3Transaction = void 0;
const sdk_1 = require("@sqds/sdk");
const bn_js_1 = __importDefault(require("bn.js"));
const web3_js_1 = require("@solana/web3.js");
const spl_utils_1 = require("@render-foundation/spl-utils");
const executeSqV3Transaction = (connection, squadsMsKey, clients, instrs) => __awaiter(void 0, void 0, void 0, function* () {
    const msAuthorityK = (0, sdk_1.getAuthorityPDA)(squadsMsKey, new bn_js_1.default(1), sdk_1.DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let txPDA;
    const primClient = clients[0];
    const txIdx = yield primClient.getNextTransactionIndex(squadsMsKey);
    txPDA = (0, sdk_1.getTxPDA)(squadsMsKey, new bn_js_1.default(txIdx), sdk_1.DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let createTx = new web3_js_1.Transaction()
        .add(yield primClient.buildCreateTransaction(squadsMsKey, 1, txIdx));
    let instrIdx = 1;
    for (const i of instrs) {
        createTx.add(yield primClient.buildAddInstruction(squadsMsKey, txPDA, i, instrIdx));
        instrIdx += 1;
    }
    createTx.add(yield primClient.buildActivateTransaction(squadsMsKey, txPDA));
    for (const c of clients) {
        createTx.add(yield c.buildApproveTransaction(squadsMsKey, txPDA));
    }
    createTx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
    createTx.feePayer = primClient.wallet.publicKey;
    for (const c of clients) {
        createTx = yield c.wallet.signTransaction(createTx);
    }
    //createTx.sign(sharedMsKp, authorityKp)
    yield (0, spl_utils_1.sendAndConfirmWithRetry)(connection, createTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    console.log(`created tx ${txPDA}`);
    console.log(`executing tx ${txPDA}`);
    let execTx = new web3_js_1.Transaction().add(yield primClient.buildExecuteTransaction(txPDA, primClient.wallet.publicKey));
    execTx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
    execTx.feePayer = primClient.wallet.publicKey;
    execTx = yield primClient.wallet.signTransaction(execTx);
    //execTx.sign(args.solFeePayerKp)
    const { txid } = yield (0, spl_utils_1.sendAndConfirmWithRetry)(connection, execTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'confirmed');
    console.log(`executed ${txid}`);
    return { txSig: txid, txPDA };
});
exports.executeSqV3Transaction = executeSqV3Transaction;
//# sourceMappingURL=squadsV3.js.map