export const renderClient = (authToken) => {
    const host = `rndr.x.io`;
    return {
        async fetchNodeOperatorRewards(startTime, endTime) {
            const startTimeStr = startTime.toISOString().split('T')[0];
            const endTimeStr = endTime.toISOString().split('T')[0];
            const url = `https://${host}/api/v3/services/solana/payments?finished_after=${startTimeStr}&finished_before=${endTimeStr}`;
            console.log(`fetching ${url}`);
            const res = await fetch(url, { headers: {
                    Authorization: `${authToken}`,
                    Accept: 'application/vnd.api+json'
                } }).then(async (res) => {
                if (res.status != 200) {
                    throw new Error(`failed fetching node operator rewards. code: ${res.status} body: ${JSON.stringify(await res.text())}`);
                }
                return (await res.json());
            });
            const solKeyToDue = {};
            for (const r of res.data.attributes.paymentsOwed) {
                const [addr, amt] = Object.entries(r)[0];
                solKeyToDue[addr] = amt;
            }
            return { solKeyToDue, jobIds: res.meta.jobIds };
        },
        async markJobsPayed(ids, finishedAfter, finishedBefore, totalDistributed) {
            const payload = {
                data: {
                    attributes: {
                        finished_after: finishedAfter.toISOString().split('T')[0],
                        finished_before: finishedBefore.toISOString().split('T')[0],
                        job_ids: ids,
                        total_distributed: totalDistributed
                    }
                }
            };
            const res = await fetch(`https://${host}/api/v3/services/solana/payments`, {
                method: 'POST',
                headers: {
                    'Authorization': `${authToken}`,
                    'Content-Type': `application/json`
                },
                body: JSON.stringify(payload)
            }).then(async (res) => {
                if (res.status != 201) {
                    throw new Error(`failed posting node operator rewards. code: ${res.status} body: ${JSON.stringify(await res.text())}`);
                }
                return (await res.text());
            });
            console.log(`markJobsPayed ${res}`);
            return;
        }
    };
};
//# sourceMappingURL=renderClient.js.map