import { DEFAULT_MULTISIG_PROGRAM_ID, getAuthorityPDA, getTxPDA } from "@sqds/sdk";
import BN from "bn.js";
import { Transaction } from "@solana/web3.js";
import { sendAndConfirmWithRetry } from "@render-foundation/spl-utils";
export const executeSqV3Transaction = async (connection, squadsMsKey, clients, instrs) => {
    const msAuthorityK = getAuthorityPDA(squadsMsKey, new BN(1), DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let txPDA;
    const primClient = clients[0];
    const txIdx = await primClient.getNextTransactionIndex(squadsMsKey);
    txPDA = getTxPDA(squadsMsKey, new BN(txIdx), DEFAULT_MULTISIG_PROGRAM_ID)[0];
    let createTx = new Transaction()
        .add(await primClient.buildCreateTransaction(squadsMsKey, 1, txIdx));
    let instrIdx = 1;
    for (const i of instrs) {
        createTx.add(await primClient.buildAddInstruction(squadsMsKey, txPDA, i, instrIdx));
        instrIdx += 1;
    }
    createTx.add(await primClient.buildActivateTransaction(squadsMsKey, txPDA));
    for (const c of clients) {
        createTx.add(await c.buildApproveTransaction(squadsMsKey, txPDA));
    }
    createTx.recentBlockhash = (await connection.getLatestBlockhash()).blockhash;
    createTx.feePayer = primClient.wallet.publicKey;
    for (const c of clients) {
        createTx = await c.wallet.signTransaction(createTx);
    }
    //createTx.sign(sharedMsKp, authorityKp)
    await sendAndConfirmWithRetry(connection, createTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'finalized');
    console.log(`created tx ${txPDA}`);
    console.log(`executing tx ${txPDA}`);
    let execTx = new Transaction().add(await primClient.buildExecuteTransaction(txPDA, primClient.wallet.publicKey));
    execTx.recentBlockhash = (await connection.getLatestBlockhash()).blockhash;
    execTx.feePayer = primClient.wallet.publicKey;
    execTx = await primClient.wallet.signTransaction(execTx);
    //execTx.sign(args.solFeePayerKp)
    const { txid } = await sendAndConfirmWithRetry(connection, execTx.serialize(), {
        skipPreflight: true,
        maxRetries: 10,
    }, 'confirmed');
    console.log(`executed ${txid}`);
    return { txSig: txid, txPDA };
};
//# sourceMappingURL=squadsV3.js.map