import type { ColumnType } from "kysely";
export type Generated<T> = T extends ColumnType<infer S, infer I, infer U> ? ColumnType<S, I | undefined, U> : ColumnType<T, T | undefined, T>;
export type Int8 = ColumnType<string, bigint | number | string, bigint | number | string>;
export type Timestamp = ColumnType<Date, Date | string, Date | string>;
export type TransferType = "node_operator_rewards" | "open_compute_rewards" | "upgrade_rewards";
export type TxStatus = "created" | "executed" | "ready";
export interface JobId {
    epoch_id: Generated<Int8>;
    id: Int8;
}
export interface NodeOperatorEpoch {
    end_time: Timestamp | null;
    id: Generated<Int8>;
    saved_at: Timestamp | null;
    start_time: Timestamp | null;
}
export interface NodeOperatorRewardsTransfer {
    end_time: Timestamp | null;
    epoch_id: Generated<Int8>;
    start_time: Timestamp | null;
    transfer_id: Generated<Int8>;
}
export interface OpenComputeRewardsTransfer {
    device_id: string | null;
    epoch_id: Generated<Int8>;
    transfer_id: Generated<Int8>;
}
export interface SquadsTx {
    id: Generated<Int8>;
    sq_tx: string;
    tx_status: TxStatus;
}
export interface Transfer {
    amount_due: Int8;
    amount_payed: Int8;
    created_at: Generated<Timestamp>;
    id: Generated<Int8>;
    payed_at: Timestamp | null;
    sol_key: string | null;
    sol_tx: string | null;
    sq_tx_id: Int8 | null;
    transfer_type: TransferType;
}
export interface UpgradeRewardsTransfer {
    epoch_id: Generated<Int8>;
    epoch_start_time: Timestamp | null;
    points: Int8 | null;
    transfer_id: Generated<Int8>;
}
export interface DB {
    job_id: JobId;
    node_operator_epoch: NodeOperatorEpoch;
    node_operator_rewards_transfer: NodeOperatorRewardsTransfer;
    open_compute_rewards_transfer: OpenComputeRewardsTransfer;
    squads_tx: SquadsTx;
    transfer: Transfer;
    upgrade_rewards_transfer: UpgradeRewardsTransfer;
}
//# sourceMappingURL=dbTypes.d.ts.map